(* This file is the only one you should modify *)
open Ast

exception Unimplemented

let all_splits s =
  let len = String.length s in
  let rec loop i =
    if i = len
    then [(s,"")]
    else (Str.string_before s i, Str.string_after s i)::(loop (i+1)) in
  loop 0

let rec interp r s = 
  match r with
    Char c -> String.length s = 1 && String.get s 0 = c
  | Sequence(r1,r2) -> 
      List.exists (fun (s1,s2) -> interp r1 s1 && interp r2 s2)
	(all_splits s)
  | Alternative(r1,r2) -> interp r1 s || interp r2 s
  | Star(r1) -> 
      String.length s = 0 || 
      List.exists (fun (s1,s2) -> interp r1 s1 && interp r s2)
	(List.tl (all_splits s))

let rec denote r = 
  match r with
    Char c -> (fun s -> String.length s = 1 && String.get s 0 = c)
  | Sequence(r1,r2) ->
      let d1 = denote r1 in
      let d2 = denote r2 in
      (fun s -> List.exists (fun (s1,s2) -> d1 s1 && d2 s2) (all_splits s))
  | Alternative(r1,r2) ->
      let d1 = denote r1 in
      let d2 = denote r2 in
      (fun s -> d1 s || d2 s)
  | Star(r1) ->
      let d1 = denote r1 in
      let rec f s =
	String.length s = 0 ||
	List.exists (fun (s1,s2) -> d1 s1 && f s2)
	(List.tl (all_splits s))
      in
      f

(* the next two bindings really belong in a "main" module, but this
keeps all the code you need in one place *)

(* expect one command-line argument, a file to parse and interpret *)
(* you do not need to understand this interaction with the system *)
let get_prog () =
  let argv = Sys.argv in
  let _ = 
    if Array.length argv != 2
    then (prerr_string ("usage: " ^ argv.(0) ^ " [file-to-interpret]\n");
	  exit 1) in
  let ch = open_in argv.(1) in
  Parse.program Lex.lexer (Lexing.from_channel ch)

let _ =
  let (r,s) = get_prog () in
  ()
  ; print_string ("ans = " ^ string_of_bool (interp r s) ^"\n")
  ; print_string ("ans = " ^ string_of_bool (denote r s) ^"\n")
